// RACE X
//
// Written by Mauricio Teichmann Ritter
//
// Copyright (C) 2002, Brazil. All rights reserved.
// 
//

#include ".\AppLib\cSprite.h"
#include ".\applib\chitchecker.h"
#include ".\applib\csound.h"

#include <vector>
#include <string>

using namespace std;

#if !defined(AFX_CRACECAR_H__599011FD_F0E9_4C8A_AAC5_ECA4A9FF481B__INCLUDED_)
#define AFX_CRACECAR_H__599011FD_F0E9_4C8A_AAC5_ECA4A9FF481B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define		CTRL_COMPUTER		1
#define		CTRL_USER			2
#define		CTRL_NETWORK_REMOTE	3
#define		CTRL_NETWORK_LOCAL	4

#define     CARSTATE_OK				1
#define     CARSTATE_CRASHED_WALL	2
#define     CARSTATE_CRASHED_CAR	3
#define     CARSTATE_RACECOMPLETED	4

typedef vector<long> LONGVECTOR;

// cRaceCar
//
// This class defines each one of the cars that makes part of a game
// Each car have its own characteristics, like color, id and player name

class cRaceCar  
{
private:
	int iTurnCar;

	// Time in which the car crashed
	int iCrashTime;

	// Distance to the Next CheckPoint (used to defined the position)
	int m_iDistanceToNextCheckPoint;

	// Position of the car in the current Race
	int m_iPosition;

	// Sound Buffers
	cSound			m_pSound;
	cSound			m_pCrashSound;

	// Vector that stores all the lap times
	LONGVECTOR	m_vcLapTimes;
	long		m_lCurrentTime;

	// These are backup vaariables used in case the car crash.
	// If the car crash we will reset the car to this Backup Variables

	int m_fBackupIncrementX;
	int m_fBackupIncrementY;
	int m_fBackupDirectionY;
	int m_fBackupDirectionX;
	int m_iBackupAngle;
	float m_fBackupPosY;
	float m_fBackupPosX;
	int iBackupTurnCar;
	int m_iBackupDistanceToNextCheckPoint;

	// Car State
	int m_iCarState;

	void TurnCarRight();
	void TurnCarLeft();

	// How the car is controlled (by the computer, by the user, by the network)
	int m_iControlType;

	// How much we should increment in X and Y position
	int m_fIncrementX;
	int m_fIncrementY;

	// Car current speed
	int m_iSpeed;

	// Maximum Speed of the Car
	int m_iMaxSpeed;

	// Current Position of the car in the track Map
	float m_fPosY;
	float m_fPosX;

	// Last time the car turned
	long m_lLastTurn;

	// Last time the car accelerated
	long m_lLastSpeedIncr;

	// Indicates where the X and Y should head
	int m_fDirectionY;
	int m_fDirectionX;



	POINT	*m_pBound;

public:
	void MoveForward(int iSpeed);
	void Backup();

	DWORD m_dwElapseTime;
	void AddLapTime(DWORD dwTime);
	void SetSpeed(int iValue);
	BYTE m_bLastSent_Speed;
	BYTE m_bLastSent_Position;
	BYTE m_bLastSent_CarState;
	BYTE m_bLastSent_Laps;
	BYTE m_bLastSent_Angle;
	DWORD m_dwLastSent_LastLapTime;
	unsigned short m_nLastSent_PosX;
	unsigned short m_nLastSent_PosY;

	BYTE m_bRemoteKeyboardStatus;
	int m_iAngle;

	BYTE GetID();
	void SetID(BYTE iValue);

	BYTE m_bID;
	int GetCarColor();
	long GetLastLapTime();
	long GetLapElapseTime(int iLap = -1);
	int GetDistanceToNextCheckPoint();
	void SetPosition(int iPosition);
	int GetPosition();
	int m_iLaps;

	int m_iBackupSprite;
	void Crashed();
	int GetCarState();
	void SetCarState(int iState);
	void Idle();
	void HitCar();
	void BreakCar();
	void Accelerate();
	int GetControlType();
	void SetControlType(int iType);
	int GetSpeed();
	int m_iCheckPoint;
	int GetLastCheckPoint();
	void SetLastCheckPoint(int iLastCheckPoint);
	cHitChecker m_hcRaceCar;
	void GetPos(int* iX, int* iY);
	void Process(void* pTrack);
	void Destroy();
	void Create(int iColor);
	void Draw(int nViewX, int nViewY);

	void SetPos(int iX, int iY);
	cSprite m_sprCar;

	cSprite m_sprCarExplode_0;
	cSprite m_sprCarExplode_45;
	cSprite m_sprCarExplode_90;
	cSprite m_sprCarExplode_135;
	cSprite m_sprCarExplode_180;
	cSprite m_sprCarExplode_225;
	cSprite m_sprCarExplode_270;
	cSprite m_sprCarExplode_315;


	cRaceCar();
	virtual ~cRaceCar();

protected:
	int m_iColor;
	void RotateBound(int iAngle);
};

#endif // !defined(AFX_CRACECAR_H__599011FD_F0E9_4C8A_AAC5_ECA4A9FF481B__INCLUDED_)
