// RACE X
//
// Written by Mauricio Teichmann Ritter
//
// Copyright (C) 2002, Brazil. All rights reserved.
// 
//

#if !defined(AFX_CRACETRACK_H__2022F72B_F3CD_47F7_904E_11F2297090EE__INCLUDED_)
#define AFX_CRACETRACK_H__2022F72B_F3CD_47F7_904E_11F2297090EE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <string>

using namespace std;

#include ".\AppLib\cSurface.h"
#include ".\AppLib\cHitChecker.h"
#include ".\AppLib\cMatrix.h"

#include "cRaceCar.h"

#define TURN_LEFT	10
#define TURN_RIGHT  20

#define TILE_SIZE   200

// cRaceTrack Class
//
// This class is used to control the tracks in the game. Its responsible for loading,
// drawing, hitchecking and controlling of the cars that are currently racing.

class cRaceTrack  
{
private:
	
	// Number of cars currently racing in this Track
	int m_iNumCars;

	// Number of Columns and Rows of the track. Each column have a 40x40 tile
	// associated with it
	int m_iCols;
	int m_iRows;

	// With portion of hte Track the user is currently looking at (offset)
	int m_iViewY;
	int m_iViewX;

	// Number of Laps needed to complete the track
	int	m_iLaps;

	// Name of the track
	char *m_sTrackName;


	// This state variable is used to know if the race started or not
	// Its used to display the semaphore in the screen when the race is starting
	// If its value is 0 we show the first semaphore pic, if 1 we show the secod
	// pic, when it reaches 2 we dont draw the semaphore anymore (if the m_lStartTime elapses)
	int m_iState;
	long m_lStartTime;

	// Which tile we are using (desert or grass)
	UINT m_Tile;

	
	// Set of surfaces used in the Track

	// This surface hold the tile picture
	cSurface	m_surfTile;

	// This surface holds de Race Completed banner
	cSurface m_surfRaceCompleted;

	
	// All this surfaces are used to draw the track. Each one represent
	// a tile of the track
	cSurface m_surfDiagonalQ0;
	cSurface m_surfDiagonalQ1;
	cSurface m_surfDiagonalQ2;
	cSurface m_surfDiagonalQ3;

	cSurface m_surfMidDiagonalQ0;
	cSurface m_surfMidDiagonalQ1;
	cSurface m_surfMidDiagonalQ2;
	cSurface m_surfMidDiagonalQ3;

	cSurface m_surfBlackPointQ0;
	cSurface m_surfBlackPointQ1;
	cSurface m_surfBlackPointQ2;
	cSurface m_surfBlackPointQ3;

	cSurface m_surf_HZ_StartDiagonalQ0;
	cSurface m_surf_HZ_StartDiagonalQ1;
	cSurface m_surf_HZ_StartDiagonalQ2;
	cSurface m_surf_HZ_StartDiagonalQ3;

	cSurface m_surf_HZ_EndDiagonalQ0;
	cSurface m_surf_HZ_EndDiagonalQ1;
	cSurface m_surf_HZ_EndDiagonalQ2;
	cSurface m_surf_HZ_EndDiagonalQ3;

	cSurface m_surf_VR_StartDiagonalQ0;
	cSurface m_surf_VR_StartDiagonalQ1;
	cSurface m_surf_VR_StartDiagonalQ2;
	cSurface m_surf_VR_StartDiagonalQ3;

	cSurface m_surf_VR_EndDiagonalQ0;
	cSurface m_surf_VR_EndDiagonalQ1;
	cSurface m_surf_VR_EndDiagonalQ2;
	cSurface m_surf_VR_EndDiagonalQ3;


	cSurface m_surfSRaceRoadQ0;
	cSurface m_surfSRaceRoadQ1;
	cSurface m_surfSRaceRoadQ2;
	cSurface m_surfSRaceRoadQ3;

	cSurface m_surfHalfRoadQ0;
	cSurface m_surfHalfRoadQ1;
	cSurface m_surfHalfRoadQ2;
	cSurface m_surfHalfRoadQ3;

	cSurface m_surfFullRoadQ0;
	cSurface m_surfFullRoadQ1;
	cSurface m_surfFullRoadQ2;
	cSurface m_surfFullRoadQ3;
	cSurface m_surfFullRoadQ4;
	cSurface m_surfFullRoadQ5;
	cSurface m_surfFullRoadQ6;
	cSurface m_surfFullRoadQ7;
	cSurface m_surfFullRoadQ8;
	cSurface m_surfFullRoadQ9;
	cSurface m_surfFullRoadQ10;
	cSurface m_surfFullRoadQ11;
	cSurface m_surfFullRoadQ12;
	cSurface m_surfFullRoadQ13;
	cSurface m_surfFullRoadQ14;
	cSurface m_surfFullRoadQ15;
	cSurface m_surfFullRoadQ16;
	cSurface m_surfFullRoadQ17;

	cSurface m_surfCurveQ0;
	cSurface m_surfCurveQ1;
	cSurface m_surfCurveQ2;
	cSurface m_surfCurveQ3;

	cSurface m_surfEndCurveQ0;
	cSurface m_surfEndCurveQ1;
	cSurface m_surfEndCurveQ2;
	cSurface m_surfEndCurveQ3;

	cSurface m_surfMediumCurveQ0P1;
	cSurface m_surfMediumCurveQ0P2;
	cSurface m_surfMediumCurveQ0P3;

	cSurface m_surfMediumCurveQ1P1;
	cSurface m_surfMediumCurveQ1P2;
	cSurface m_surfMediumCurveQ1P3;

	cSurface m_surfMediumCurveQ2P1;
	cSurface m_surfMediumCurveQ2P2;
	cSurface m_surfMediumCurveQ2P3;

	cSurface m_surfMediumCurveQ3P1;
	cSurface m_surfMediumCurveQ3P2;
	cSurface m_surfMediumCurveQ3P3;
	
	cSurface m_surfEndMediumCurveQ0P1;
	cSurface m_surfEndMediumCurveQ0P2;
	cSurface m_surfEndMediumCurveQ0P3;

	cSurface m_surfEndMediumCurveQ1P1;
	cSurface m_surfEndMediumCurveQ1P2;
	cSurface m_surfEndMediumCurveQ1P3;

	cSurface m_surfEndMediumCurveQ2P1;
	cSurface m_surfEndMediumCurveQ2P2;
	cSurface m_surfEndMediumCurveQ2P3;

	cSurface m_surfEndMediumCurveQ3P1;
	cSurface m_surfEndMediumCurveQ3P2;
	cSurface m_surfEndMediumCurveQ3P3;

	// This function is used to remove the polygonal region of each tile
	// from the main HitCheching region of the track
	void RemoveHitRegion();

public:
	// Public variables

	// This sprite will hold the semaphore animation
	cSprite		m_sptrWait;

	// This is an array cRaceCars. It represent all the cars that are
	// currently racing in the track
	// For each race we need to add the set of cars to the cRaceTrack class
	// to start the race
	cRaceCar*   m_pRaceCars[4];

	// This Matrix hold the tiles information
	cMatrix		m_RoadMap;

	// This is responsible to hold the valid region of the track. If the
	// car hits an invalid region it will crash
	cHitChecker	m_hcRoadMap;

	// Sound buffer of the semaphore
	cSound		m_sndSemaphore;

	
	BYTE GetLocalCarID();
	void SetRemoteKeyboardStatus(BYTE bIdCar, BYTE bStatus);
	int GetAngle(int iCol, int iRow);
	void SetCarInfo(BYTE bID, BYTE bMask, 
					DWORD   dwElapseTime,
					BYTE	bLaps,
					int	    iSpeed,
					BYTE	bCarPosition,
					DWORD	dwLastLapTime,
					unsigned short nCarX,
					unsigned short nCarY,
					BYTE	bAngle,
					BYTE	bCarState);

	char* GetFileName();
	string m_sFileName;
	int GetHeight();
	int GetWidth();
	cRaceCar* GetCar(int iPos);
	int GetNumberOfCars();
	int GetNumberOfLaps();
	char* GetTrackName();
	void GetCheckPointPosition(int *iX, int* iY,int iCheckPoint);
//	int TurnDirection(int iX, int iY, int iAngle);
	int GetRoadType(int iX, int iY);
	void AddCar(cRaceCar* pCar);
	void Process();

	int m_iMaxCheckPoint;
	int GetMaxCheckPoint();
	int GetCheckPoint(int iX, int iY);
	BOOL CarHittedRoad(cRaceCar* pCar, int iX = -1, int iY = -1);
	int GetViewY();
	int GetViewX();
	void AdjustView(cRaceCar* pPar);
	void GetStartPosition(int* iX, int* iY);
	
	BOOL ReadFromFile(char* lpszFile);
	void SetRoad(int iCol, int iRow, int iType);


	void Draw(int iDestX = 0, int iDestY = 0, int nX =-1 , int nY =-1 , int nWidth = 640, int nHeight = 480);
	void Destroy(BOOL bLastDestroy = FALSE);
	//cSurface m_surfMap;

	void SetTile(UINT iTile);
	void Create(int nWidth, int nHeight);


	cRaceTrack();
	virtual ~cRaceTrack();

};

#endif // !defined(AFX_CRACETRACK_H__2022F72B_F3CD_47F7_904E_11F2297090EE__INCLUDED_)
